@@ -21,7 +21,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -230,7 +230,7 @@ Don't change without re-evaluating the file.")
 (put 'cg-indentation 'safe-local-variable 'integerp)
 
 (defconst cg-font-lock-keywords-1
-  (let ((<word>? "\\(?:\"<[^>]+>\"\\)?"))
+  (let ((<word>? "\\(?:\"<[^>]+>\"[irv]*\\)?"))
     `((,(format "^[ \t]*\\(%s\\)[ \t]+\\(\\(\\sw\\|\\s_\\)+\\)" cg-kw-set-re)
        (1 font-lock-keyword-face)
        (2 font-lock-variable-name-face))
@@ -1113,6 +1113,7 @@ Use 0 to check immediately after each change."
 (defcustom cg-output-always-highlight nil
   "Regexp patterns of tags to always highlight in output, with a face to use."
   :local t
+  :safe (lambda (_) t)
   :type '(repeat (cons regexp face)))
 
 (defun cg-output-hl-always-patterns ()
@@ -1199,8 +1200,14 @@ Similarly, `cg-post-pipe' is run on output."
        (post-pipe (if (and cg-post-pipe (not (equal "" cg-post-pipe)))
                       (concat " | " cg-post-pipe)
                     ""))
+       ;; cg-proc doesn't expect the --grammar argument (a bit hacky,
+       ;; better would be to make cg-extra-args a function taking
+       ;; grammar path as string):
+       (grammar-arg (if (string-match-p ".*cg-proc$" cg-command)
+                        ""
+                      "--grammar"))
        (cmd (concat
-             cg-command " " cg-extra-args " --grammar " tmp
+             cg-command " " cg-extra-args " " grammar-arg " " tmp " "
              post-pipe))
        (in (cg--get-input-buffer file))
        (out (progn (write-region (point-min) (point-max) tmp)
