@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "BinaryGrammar.hpp"
@@ -239,6 +237,10 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
 			t->variable_hash = ntoh32(u32tmp);
 		}
+		if (fields & (1 << 14)) {
+			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
+			t->context_ref_pos = ntoh32(u32tmp);
+		}
 
 		grammar->single_tags[t->hash] = t;
 		grammar->single_tags_list[t->number] = t;
@@ -318,7 +320,10 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
 			s->number = ntoh32(u32tmp);
 		}
-		// Field 1 is unused
+		if (fields & (1 << 1)) {
+			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
+			s->type = UI16(ntoh32(u32tmp));
+		}
 		if (fields & (1 << 2)) {
 			fread_throw(&u8tmp, sizeof(u8tmp), 1, input);
 			s->type = u8tmp;
@@ -434,8 +439,12 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 			r->line = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 3)) {
-			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
-			r->flags = ntoh32(u32tmp);
+			if (fields & (1 << 16)) {
+				r->flags = readSwapped<uint64_t>(input);
+			}
+			else {
+				r->flags = readSwapped<uint32_t>(input);
+			}
 		}
 		if (fields & (1 << 4)) {
 			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
@@ -520,6 +529,36 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 			ContextualTest* t = grammar->contexts[u32tmp];
 			r->addContextualTest(t, r->tests);
 		}
+
+		if (fields & (1 << 15)) {
+			fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
+			u32tmp = ntoh32(u32tmp);
+			uint32_t num_sub_rules = u32tmp;
+			for (uint32_t j = 0; j < num_sub_rules; j++) {
+				fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
+				u32tmp = ntoh32(u32tmp);
+				r->sub_rules.push_back(grammar->rule_by_number[u32tmp]);
+			}
+		}
+
+		UErrorCode status;
+		if (nrules) {
+			status = U_ZERO_ERROR;
+			uregex_setText(nrules, r->name.c_str(), SI32(r->name.size()), &status);
+			status = U_ZERO_ERROR;
+			if (!uregex_find(nrules, -1, &status)) {
+				r->type = K_IGNORE;
+			}
+		}
+		if (nrules_inv) {
+			status = U_ZERO_ERROR;
+			uregex_setText(nrules_inv, r->name.c_str(), SI32(r->name.size()), &status);
+			status = U_ZERO_ERROR;
+			if (uregex_find(nrules_inv, -1, &status)) {
+				r->type = K_IGNORE;
+			}
+		}
+
 		grammar->rule_by_number[r->number] = r;
 	}
 
@@ -545,6 +584,7 @@ ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
 	uint32_t fields = 0;
 	uint32_t u32tmp = 0;
 	int32_t i32tmp = 0;
+	int8_t i8tmp = 0;
 	uint32_t tmpl = 0;
 
 	fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
@@ -595,6 +635,10 @@ ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
 		fread_throw(&i32tmp, sizeof(i32tmp), 1, input);
 		t->offset_sub = SI32(ntohl(i32tmp));
 	}
+	if (fields & (1 << 12)) {
+		fread_throw(&i8tmp, sizeof(i8tmp), 1, input);
+		t->jump_pos = i8tmp;
+	}
 	if (fields & (1 << 10)) {
 		fread_throw(&u32tmp, sizeof(u32tmp), 1, input);
 		uint32_t num_ors = ntoh32(u32tmp);
