@@ -3,20 +3,18 @@
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "BinaryGrammar.hpp"
@@ -470,6 +468,25 @@ int BinaryGrammar::readBinaryGrammar_10043(std::istream& input) {
 			ContextualTest* t = contexts_list[u32tmp - 1];
 			r->addContextualTest(t, r->tests);
 		}
+
+		UErrorCode status;
+		if (nrules) {
+			status = U_ZERO_ERROR;
+			uregex_setText(nrules, r->name.c_str(), SI32(r->name.size()), &status);
+			status = U_ZERO_ERROR;
+			if (!uregex_find(nrules, -1, &status)) {
+				r->type = K_IGNORE;
+			}
+		}
+		if (nrules_inv) {
+			status = U_ZERO_ERROR;
+			uregex_setText(nrules_inv, r->name.c_str(), SI32(r->name.size()), &status);
+			status = U_ZERO_ERROR;
+			if (uregex_find(nrules_inv, -1, &status)) {
+				r->type = K_IGNORE;
+			}
+		}
+
 		grammar->rule_by_number[r->number] = r;
 	}
 
