@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "BinaryGrammar.hpp"
@@ -196,10 +194,14 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 			}
 		}
 		// 1 << 12 used above
-		if ((t->type & T_VARIABLE) && t->variable_hash) {
+		if ((t->type & (T_VARIABLE|T_LOCAL_VARIABLE)) && t->variable_hash) {
 			fields |= (1 << 13);
 			writeSwapped(buffer, t->variable_hash);
 		}
+		if (t->type & T_CONTEXT) {
+			fields |= (1 << 14);
+			writeSwapped(buffer, t->context_ref_pos);
+		}
 
 		u32tmp = hton32(fields);
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
@@ -260,10 +262,13 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 			fields |= (1 << 0);
 			writeSwapped(buffer, s->number);
 		}
-		// Field 1 is unused
-		if (s->type) {
+		if (s->type >= ST_ORDERED) {
+			fields |= (1 << 1);
+			writeSwapped(buffer, UI32(s->type));
+		}
+		else {
 			fields |= (1 << 2);
-			writeSwapped(buffer, s->type);
+			writeSwapped(buffer, UI8(s->type));
 		}
 		if (!s->getNonEmpty().empty()) {
 			fields |= (1 << 3);
@@ -347,7 +352,13 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 		}
 		if (r->flags) {
 			fields |= (1 << 3);
-			writeSwapped(buffer, r->flags);
+			if (r->flags > std::numeric_limits<uint32_t>::max()) {
+				fields |= (1 << 16);
+				writeSwapped(buffer, r->flags);
+			}
+			else {
+				writeSwapped(buffer, UI32(r->flags));
+			}
 		}
 		if (!r->name.empty()) {
 			fields |= (1 << 4);
@@ -400,6 +411,9 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 			fields |= (1 << 14);
 			writeSwapped(buffer, r->number);
 		}
+		if (!r->sub_rules.empty()) {
+			fields |= (1 << 15);
+		}
 
 		u32tmp = hton32(fields);
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
@@ -426,6 +440,15 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 			u32tmp = hton32(it->hash);
 			fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 		}
+
+		if (!r->sub_rules.empty()) {
+			u32tmp = hton32(static_cast<unsigned long>(r->sub_rules.size()));
+			fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
+			for (auto it : r->sub_rules) {
+				u32tmp = hton32(it->number);
+				fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
+			}
+		}
 	}
 
 	ucnv_close(conv);
@@ -505,6 +528,10 @@ void BinaryGrammar::writeContextualTest(ContextualTest* t, FILE* output) {
 	if (t->linked) {
 		fields |= (1 << 11);
 	}
+	if (t->jump_pos) {
+		fields |= (1 << 12);
+		writeSwapped(buffer, t->jump_pos);
+	}
 
 	u32tmp = hton32(fields);
 	fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
