@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -73,6 +71,7 @@ public:
 	boost::dynamic_bitset<> possible_sets;
 	CohortVector enclosed;
 	CohortVector removed;
+	CohortVector ignored_cohorts;
 	RelationCtn relations;
 	RelationCtn relations_input;
 	uint32_t line_number = 0;
@@ -117,7 +116,7 @@ typedef sorted_vector<Cohort*, compare_Cohort> CohortSet;
 typedef std::unordered_map<uint32_t, CohortSet> uint32ToCohortsMap;
 
 Cohort* alloc_cohort(SingleWindow* p);
-void free_cohort(Cohort* c);
+void free_cohort(Cohort*& c);
 }
 
 #endif
