@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -56,7 +54,7 @@ enum : uint64_t {
 	POS_ALL            = (1 << 18), // Prefix ALL
 	POS_DEP_DEEP       = (1 << 19), // * or **
 	POS_MARK_SET       = (1 << 20), // X
-	POS_MARK_JUMP      = (1 << 21), // x
+	POS_JUMP           = (1 << 21), // x, jM, jA, jT, jCn
 	POS_LOOK_DELETED   = (1 << 22), // D
 	POS_LOOK_DELAYED   = (1 << 23), // d
 	POS_TMPL_OVERRIDE  = (1 << 24),
@@ -85,6 +83,13 @@ enum : uint64_t {
 	MASK_SELF_NB       = POS_SELF | POS_NO_BARRIER,
 };
 
+enum POS_JUMP_POS : int8_t {
+	JUMP_TARGET = -2,
+	JUMP_ATTACH = -1,
+	JUMP_MARK = 0,
+	// All positive numbers are WITH's cohorts
+};
+
 enum GSR_SPECIALS {
 	GSR_ANY = 32767,
 };
@@ -102,8 +107,7 @@ public:
 	uint32_t relation = 0;
 	uint32_t barrier = 0;
 	uint32_t cbarrier = 0;
-	mutable uint32_t num_fail = 0, num_match = 0;
-	mutable double total_time = 0;
+	int8_t jump_pos = JUMP_MARK;
 	ContextualTest* tmpl = nullptr;
 	ContextualTest* linked = nullptr;
 
@@ -114,7 +118,6 @@ public:
 	bool operator==(const ContextualTest&) const;
 	bool operator!=(const ContextualTest& o) const { return !(*this == o); }
 	uint32_t rehash();
-	void resetStatistics();
 	void markUsed(Grammar& grammar);
 };
 
@@ -129,6 +132,7 @@ inline void copy_cntx(const ContextualTest* src, ContextualTest* trg) {
 	trg->relation = src->relation;
 	trg->barrier = src->barrier;
 	trg->cbarrier = src->cbarrier;
+	trg->jump_pos = src->jump_pos;
 	trg->tmpl = src->tmpl;
 	trg->linked = src->linked;
 }
