@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "FormatConverter.hpp"
@@ -73,26 +71,26 @@ void FormatConverter::runGrammarOnText(std::istream& input, std::ostream& output
 	}
 }
 
-void FormatConverter::printCohort(Cohort* cohort, std::ostream& output) {
+void FormatConverter::printCohort(Cohort* cohort, std::ostream& output, bool profiling) {
 	switch (outformat) {
 	case FMT_CG: {
-		GrammarApplicator::printCohort(cohort, output);
+		GrammarApplicator::printCohort(cohort, output, profiling);
 		break;
 	}
 	case FMT_APERTIUM: {
-		ApertiumApplicator::printCohort(cohort, output);
+		ApertiumApplicator::printCohort(cohort, output, profiling);
 		break;
 	}
 	case FMT_FST: {
-		FSTApplicator::printCohort(cohort, output);
+		FSTApplicator::printCohort(cohort, output, profiling);
 		break;
 	}
 	case FMT_NICELINE: {
-		NicelineApplicator::printCohort(cohort, output);
+		NicelineApplicator::printCohort(cohort, output, profiling);
 		break;
 	}
 	case FMT_PLAIN: {
-		PlaintextApplicator::printCohort(cohort, output);
+		PlaintextApplicator::printCohort(cohort, output, profiling);
 		break;
 	}
 	default:
@@ -100,26 +98,26 @@ void FormatConverter::printCohort(Cohort* cohort, std::ostream& output) {
 	}
 }
 
-void FormatConverter::printSingleWindow(SingleWindow* window, std::ostream& output) {
+void FormatConverter::printSingleWindow(SingleWindow* window, std::ostream& output, bool profiling) {
 	switch (outformat) {
 	case FMT_CG: {
-		GrammarApplicator::printSingleWindow(window, output);
+		GrammarApplicator::printSingleWindow(window, output, profiling);
 		break;
 	}
 	case FMT_APERTIUM: {
-		ApertiumApplicator::printSingleWindow(window, output);
+		ApertiumApplicator::printSingleWindow(window, output, profiling);
 		break;
 	}
 	case FMT_FST: {
-		FSTApplicator::printSingleWindow(window, output);
+		FSTApplicator::printSingleWindow(window, output, profiling);
 		break;
 	}
 	case FMT_NICELINE: {
-		NicelineApplicator::printSingleWindow(window, output);
+		NicelineApplicator::printSingleWindow(window, output, profiling);
 		break;
 	}
 	case FMT_PLAIN: {
-		PlaintextApplicator::printSingleWindow(window, output);
+		PlaintextApplicator::printSingleWindow(window, output, profiling);
 		break;
 	}
 	default:
