@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "GrammarApplicator.hpp"
@@ -389,8 +387,8 @@ Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 
 	// Replace $1-$9 with their respective match groups
 	constexpr UStringView grp[] = { STR_VS1, STR_VS2, STR_VS3, STR_VS4, STR_VS5, STR_VS6, STR_VS7, STR_VS8, STR_VS9 };
-	for (size_t i = 0; i < regexgrps.first && i < 9; ++i) {
-		findAndReplace(tmp, grp[i].data(), USV((*regexgrps.second)[i]));
+	for (size_t i = 0; i < context_stack.back().regexgrp_ct && i < 9; ++i) {
+		findAndReplace(tmp, grp[i].data(), USV((*context_stack.back().regexgrps)[i]));
 		did_something = true;
 	}
 
@@ -399,19 +397,19 @@ Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 	do {
 		found = false;
 		int32_t pos = -1, mpos = -1;
-		if ((pos = tmp.lastIndexOf(STR_VSu.data(), STR_VSu.size(), 0)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSu.data(), SI32(STR_VSu.size()), 0)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(STR_VSU.data(), STR_VSU.size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSU.data(), SI32(STR_VSU.size()), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(STR_VSl.data(), STR_VSl.size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSl.data(), SI32(STR_VSl.size()), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(STR_VSL.data(), STR_VSL.size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSL.data(), SI32(STR_VSL.size()), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
@@ -673,7 +671,7 @@ void GrammarApplicator::splitAllMappings(all_mappings_t& all_mappings, Cohort& c
 		}
 		splitMappings(iter->second, cohort, *reading, mapped);
 	}
-	std::sort(cohort.readings.begin(), cohort.readings.end(), CG3::Reading::cmp_number);
+	std::sort(cohort.readings.begin(), cohort.readings.end(), Reading::cmp_number);
 	if (!grammar->reopen_mappings.empty()) {
 		for (auto reading : cohort.readings) {
 			if (reading->mapping && grammar->reopen_mappings.count(reading->mapping->hash)) {
@@ -761,7 +759,7 @@ void GrammarApplicator::mergeReadings(ReadingList& readings) {
 		order.push_back(nr);
 	}
 
-	std::sort(order.begin(), order.end(), CG3::Reading::cmp_number);
+	std::sort(order.begin(), order.end(), Reading::cmp_number);
 	readings.insert(readings.begin(), order.begin(), order.end());
 }
 
