@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "GrammarApplicator.hpp"
@@ -31,12 +29,12 @@
 namespace CG3 {
 
 Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* test, uint8_t& rvs, bool* retval, Cohort** deep, Cohort* origin) {
-	uint8_t regexgrpz = regexgrps.first;
+	uint8_t regexgrpz = context_stack.back().regexgrp_ct;
 	if (test->pos & POS_MARK_SET) {
-		mark = cohort;
+		set_mark(cohort);
 	}
 	if (test->pos & POS_ATTACH_TO) {
-		if (attach_to != cohort) {
+		if (get_attach_to().cohort != cohort) {
 			// Clear readings for rules that care about readings
 			ReadingList* lists[4] = { &cohort->readings };
 			if (test->pos & POS_LOOK_DELETED) {
@@ -59,8 +57,6 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 				}
 			}
 		}
-
-		attach_to = cohort;
 	}
 	if (test->pos & POS_WITH) {
 		merge_with = cohort;
@@ -125,7 +121,7 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 		rvs &= ~(TRV_BREAK | TRV_BARRIER);
 	}
 	if (!*retval) {
-		regexgrps.first = regexgrpz;
+		context_stack.back().regexgrp_ct = regexgrpz;
 	}
 	return cohort;
 }
@@ -242,7 +238,7 @@ Cohort* GrammarApplicator::runContextualTest_tmpl(SingleWindow* sWindow, size_t
 	uint32_t orgbar = tmpl->barrier;
 	if (test->pos & POS_TMPL_OVERRIDE) {
 		tmpl->pos = test->pos;
-		tmpl->pos &= ~(POS_NEGATE | POS_NOT | POS_MARK_JUMP);
+		tmpl->pos &= ~(POS_NEGATE | POS_NOT | POS_JUMP);
 		tmpl->offset = test->offset;
 		if (test->offset != 0 && !(test->pos & (POS_SCANFIRST | POS_SCANALL | POS_ABSOLUTE))) {
 			tmpl->pos |= POS_SCANALL;
@@ -285,17 +281,42 @@ Cohort* GrammarApplicator::runContextualTest(SingleWindow* sWindow, size_t posit
 
 	Cohort* cohort = nullptr;
 	bool retval = true;
+	auto orgSWin = sWindow;
 
-	ticks tstamp(gtimer);
-	if (statistics) {
-		tstamp = getticks();
+	if (test->pos & POS_JUMP) {
+		Cohort* j = nullptr;
+		if (test->jump_pos == JUMP_MARK) {
+			j = get_mark();
+		}
+		else if (test->jump_pos == JUMP_ATTACH) {
+			j = get_attach_to().cohort;
+		}
+		else if (test->jump_pos == JUMP_TARGET) {
+			if (!context_stack.empty()) {
+				j = context_stack.back().target.cohort;
+			}
+		}
+		else {
+			if (context_stack.size() > 1) {
+				auto& ctx = context_stack[context_stack.size() - 2];
+				if (ctx.context.size() >= UIZ(test->jump_pos)) {
+					j = ctx.context[test->jump_pos - 1];
+				}
+			}
+		}
+		if (j) {
+			sWindow = j->parent;
+			position = j->local_number;
+		}
+		else {
+			retval = false;
+		}
 	}
-
-	if (test->pos & POS_MARK_JUMP) {
-		sWindow = mark->parent;
-		position = mark->local_number;
+	int32_t pos = SI32(position) + test->offset;
+	if (!retval) {
+		// jump failed because position does not exist
+		goto label_gotACohort;
 	}
-	int32_t pos = 0;
 
 	if (test->tmpl) {
 		Cohort* cdeep = nullptr;
@@ -520,8 +541,8 @@ Cohort* GrammarApplicator::runContextualTest(SingleWindow* sWindow, size_t posit
 			size_t seen = 0;
 			if ((test->pos & POS_SELF) && (!(test->pos & MASK_POS_LORR) || ((test->pos & POS_DEP_PARENT) && !(test->pos & POS_DEP_GLOB)))) {
 				++seen;
-				assert(pos >= 0 && pos < SI32(sWindow->cohorts.size()) && "Somehow, the input position wasn't inside the current window.");
-				Cohort* self = sWindow->cohorts[position];
+				assert(position < orgSWin->cohorts.size() && "Somehow, the input position wasn't inside the current window.");
+				Cohort* self = orgSWin->cohorts[position];
 				nc = runSingleTest(self, test, rvs, &retval, deep, origin);
 				if (!retval && (rvs & TRV_BREAK_DEFAULT)) {
 					rvs &= ~(TRV_BREAK | TRV_BREAK_DEFAULT);
@@ -577,17 +598,22 @@ label_gotACohort:
 	if (test->pos & POS_NEGATE) {
 		retval = !retval;
 	}
-	if (retval) {
-		test->num_match++;
-	}
-	else {
-		test->num_fail++;
-	}
 
-	if (statistics) {
-		ticks tmp = getticks();
-		test->total_time += elapsed(tmp, tstamp);
+	/*
+	if (profiler) {
+		auto it = profiler->contexts.find(test->hash);
+		if (it != profiler->contexts.end()) {
+			if (retval) {
+				++it->second.num_match;
+				auto rc = std::make_pair(current_rule->number + 1, test->hash);
+				++profiler->rule_contexts[rc];
+			}
+			else {
+				++it->second.num_fail;
+			}
+		}
 	}
+	//*/
 
 	if (!retval) {
 		cohort = nullptr;
@@ -780,7 +806,7 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 
 	// Recursion may happen, so can't be static
 	CohortSet rels;
-	uint8_t regexgrpz = regexgrps.first;
+	uint8_t regexgrpz = context_stack.back().regexgrp_ct;
 
 	auto rtag = grammar->single_tags[test->relation];
 	while (rtag->type & T_VARSTRING) {
@@ -805,7 +831,7 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 				auto it = sWindow->parent->cohort_map.find(citer);
 				if (it != sWindow->parent->cohort_map.end() && doesTagMatchRegexp(riter.first, *rtag, caps != 0)) {
 					rels.insert(it->second);
-					regexgrps.first = std::min(regexgrps.first, UI8(regexgrpz + caps));
+					context_stack.back().regexgrp_ct = std::min(context_stack.back().regexgrp_ct, UI8(regexgrpz + caps));
 				}
 			}
 		}
@@ -868,7 +894,7 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 	}
 
 	if (!rv) {
-		regexgrps.first = regexgrpz;
+		context_stack.back().regexgrp_ct = regexgrpz;
 	}
 	return rv;
 }
