@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -36,7 +34,14 @@ public:
 	{
 	}
 
-	virtual ~IGrammarParser() = default;
+	virtual ~IGrammarParser() {
+		if (nrules) {
+			uregex_close(nrules);
+		}
+		if (nrules_inv) {
+			uregex_close(nrules_inv);
+		}
+	}
 	virtual void setCompatible(bool compat) = 0;
 	virtual void setVerbosity(uint32_t level) = 0;
 	virtual int parse_grammar(const char* buffer, size_t length) = 0;
@@ -45,6 +50,8 @@ public:
 	virtual int parse_grammar(const char* filename) = 0;
 
 	std::ostream* ux_stderr = nullptr;
+	URegularExpression* nrules = nullptr;
+	URegularExpression* nrules_inv = nullptr;
 
 protected:
 	virtual int parse_grammar(UString& buffer) = 0;
