@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -39,6 +37,7 @@ enum {
 	ST_MAPPING     = (1 <<  5),
 	ST_USED        = (1 <<  6),
 	ST_STATIC      = (1 <<  7),
+	ST_ORDERED     = (1 <<  8),
 
 	MASK_ST_UNIFY  = ST_TAG_UNIFY | ST_SET_UNIFY | ST_CHILD_UNIFY,
 };
@@ -47,12 +46,10 @@ class Set {
 public:
 	CG3_IMPORTS static std::ostream* dump_hashes_out;
 
-	uint8_t type = 0;
+	uint16_t type = 0; // Stored as 32bit in binary format, so safe to bump when needed
 	uint32_t line = 0;
 	uint32_t hash = 0;
 	uint32_t number = 0;
-	mutable uint32_t num_fail = 0, num_match = 0;
-	mutable double total_time = 0;
 	UString name;
 
 	trie_t trie;
@@ -77,7 +74,6 @@ public:
 
 	bool empty() const;
 	uint32_t rehash();
-	void resetStatistics();
 	void reindex(Grammar& grammar);
 	void markUsed(Grammar& grammar);
 
