@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -67,15 +65,16 @@ enum : uint32_t {
 	T_ATTACHTO         = (1 << 19),
 	T_SPECIAL          = (1 << 20),
 	T_USED             = (1 << 21),
-	//T_GRAMMAR          = (1 << 22), // No longer needed since switch to grammar-owned single_tags
+	T_LOCAL_VARIABLE   = (1 << 22),
 	T_SET              = (1 << 23),
 	T_VSTR             = (1 << 24),
 	T_ENCL             = (1 << 25),
 	T_RELATION         = (1 << 26),
+	T_CONTEXT          = (1 << 27),
 
 	T_REGEXP_LINE      = (1u << 31), // ToDo: Remove for real ordered mode
 
-	MASK_TAG_SPECIAL   = T_ANY | T_TARGET | T_MARK | T_ATTACHTO | T_PAR_LEFT | T_PAR_RIGHT | T_NUMERICAL | T_VARIABLE | T_META | T_FAILFAST | T_CASE_INSENSITIVE | T_REGEXP | T_REGEXP_ANY | T_VARSTRING | T_SET | T_ENCL | T_SAME_BASIC,
+	MASK_TAG_SPECIAL   = T_ANY | T_TARGET | T_MARK | T_ATTACHTO | T_PAR_LEFT | T_PAR_RIGHT | T_NUMERICAL | T_VARIABLE | T_LOCAL_VARIABLE | T_META | T_FAILFAST | T_CASE_INSENSITIVE | T_REGEXP | T_REGEXP_ANY | T_VARSTRING | T_SET | T_ENCL | T_SAME_BASIC | T_CONTEXT,
 };
 
 class Tag {
@@ -90,12 +89,14 @@ public:
 	union {
 		uint32_t dep_parent = 0;
 		uint32_t variable_hash;
+		uint32_t context_ref_pos;
 	};
 	uint32_t hash = 0;
 	uint32_t plain_hash = 0;
 	uint32_t number = 0;
 	uint32_t seed = 0;
 	UString tag;
+	UString tag_raw;
 	std::unique_ptr<SetVector> vs_sets;
 	std::unique_ptr<UStringVector> vs_names;
 	mutable URegularExpression* regexp = nullptr;
