@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "TextualParser.hpp"
@@ -33,7 +31,7 @@ namespace CG3 {
 TextualParser::TextualParser(Grammar& res, std::ostream& ux_err, bool _dump_ast)
   : IGrammarParser(res, ux_err)
 {
-	dump_ast = _dump_ast;
+	parse_ast = _dump_ast;
 }
 
 void TextualParser::print_ast(std::ostream& out) {
@@ -41,7 +39,9 @@ void TextualParser::print_ast(std::ostream& out) {
 		return;
 	}
 	u_fprintf(out, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-	u_fprintf(out, "<!-- l is line; b is begin, e is end, both are absolute UTF-16 code unit offsets (not code point) in the file -->\n");
+	u_fprintf(out, "<!-- l is line -->\n");
+	u_fprintf(out, "<!-- b is begin, e is end - both are absolute UTF-16 code unit offsets (not code point) in the file -->\n");
+	u_fprintf(out, "<!-- u is the deduplicated objects' unique identifier -->\n");
 	::print_ast(out, ast.cs.front().b, 0, ast.cs.front());
 }
 
@@ -125,7 +125,7 @@ Tag* TextualParser::parseTag(const UChar* to, const UChar* p) {
 	}
 
 	if (!strict_tags.empty() && !strict_tags.count(tag->plain_hash)) {
-		if (tag->type & (T_ANY | T_VARSTRING | T_VSTR | T_META | T_VARIABLE | T_SET | T_PAR_LEFT | T_PAR_RIGHT | T_ENCL | T_TARGET | T_MARK | T_ATTACHTO | T_SAME_BASIC)) {
+		if (tag->type & (T_ANY | T_VARSTRING | T_VSTR | T_META | T_VARIABLE | T_LOCAL_VARIABLE | T_SET | T_PAR_LEFT | T_PAR_RIGHT | T_ENCL | T_TARGET | T_MARK | T_ATTACHTO | T_SAME_BASIC)) {
 			// Always allow...
 		}
 		else if (tag->tag == STR_BEGINTAG || tag->tag == STR_ENDTAG) {
@@ -171,7 +171,7 @@ Tag* TextualParser::addTag(Tag* tag) {
 	return result->addTag(tag);
 }
 
-void TextualParser::parseTagList(UChar*& p, Set* s) {
+void TextualParser::parseTagList(UChar*& p, Set* s, bool ordered) {
 	AST_OPEN(TagList);
 	TagVectorSet taglists;
 	bc::flat_map<Tag*, size_t> tag_freq;
@@ -202,7 +202,7 @@ void TextualParser::parseTagList(UChar*& p, Set* s) {
 					Tag* t = parseTag(&gbuffers[0][0], p);
 					tags.push_back(t);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, t->hash);
 					result->lines += SKIPWS(p, ';', ')');
 				}
 				if (*p != ')') {
@@ -228,12 +228,14 @@ void TextualParser::parseTagList(UChar*& p, Set* s) {
 				Tag* t = parseTag(&gbuffers[0][0], p);
 				tags.push_back(t);
 				p = n;
-				AST_CLOSE(p);
+				AST_CLOSE_ID(p, t->hash);
 			}
 
-			// sort + uniq the tags
-			std::sort(tags.begin(), tags.end(), compare_Tag());
-			tags.erase(std::unique(tags.begin(), tags.end(), equal_Tag()), tags.end());
+			if (!ordered) {
+				// sort + uniq the tags
+				std::sort(tags.begin(), tags.end(), compare_Tag());
+				tags.erase(std::unique(tags.begin(), tags.end(), equal_Tag()), tags.end());
+			}
 			// If this particular list of tags hasn't already been seen, then increment their frequency counts
 			if (taglists.insert(tags).second) {
 				for (auto t : tags) {
@@ -251,9 +253,11 @@ void TextualParser::parseTagList(UChar*& p, Set* s) {
 			continue;
 		}
 		TagVector& tv = const_cast<TagVector&>(tvc);
-		// Sort tags by frequency, high-to-low
-		// Doing this yields a very cheap imperfect form of trie compression, but it's good enough
-		std::sort(tv.begin(), tv.end(), fs);
+		if (!ordered) {
+			// Sort tags by frequency, high-to-low
+			// Doing this yields a very cheap imperfect form of trie compression, but it's good enough
+			std::sort(tv.begin(), tv.end(), fs);
+		}
 		bool special = false;
 		for (auto tag : tv) {
 			if (tag->type & T_SPECIAL) {
@@ -295,6 +299,7 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 					UChar* n = p;
 					++p;
 					Set* set_c = result->allocateSet();
+					set_c->type |= ST_ORDERED;
 					set_c->line = result->lines;
 					set_c->setName(sets_counter++);
 					TagVector tags;
@@ -317,14 +322,13 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 						Tag* t = parseTag(&gbuffers[0][0], p);
 						tags.push_back(t);
 						p = n;
-						AST_CLOSE(p);
+						AST_CLOSE_ID(p, t->hash);
 						result->lines += SKIPWS(p, ';', ')');
 					}
 					if (*p != ')') {
 						error("%s: Error: Expected closing ) on line %u near `%S`!\n", p);
 					}
 					++p;
-					AST_CLOSE(p);
 
 					if (tags.size() == 0) {
 						error("%s: Error: Empty inline set on line %u near `%S`! Use (*) if you want to replace with nothing.\n", n);
@@ -350,6 +354,7 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 
 					result->addSet(set_c);
 					sets.push_back(set_c->hash);
+					AST_CLOSE_ID(p, set_c->hash);
 				}
 				else {
 					AST_OPEN(SetName);
@@ -365,7 +370,7 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 					uint32_t sh = tmp->hash;
 					sets.push_back(sh);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, sh);
 				}
 
 				if (!set_ops.empty() && (set_ops.back() == S_SET_DIFF || set_ops.back() == S_SET_ISECT_U || set_ops.back() == S_SET_SYMDIFF_U)) {
@@ -461,7 +466,6 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 			error("%s: Error: Expected set on line %u near `%S`!\n", p);
 		}
 	}
-	AST_CLOSE(p);
 
 	if (!s && sets.empty()) {
 		error("%s: Error: Expected set on line %u near `%S`!\n", p);
@@ -477,6 +481,8 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 		s->sets.swap(sets);
 		s->set_ops.swap(set_ops);
 	}
+
+	AST_CLOSE_ID(p, s->hash);
 	return s;
 }
 
@@ -502,7 +508,7 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 
 	size_t tries;
 	for (tries = 0; *p != ' ' && *p != '(' && *p != '/' && tries < 100; ++tries) {
-		if (*p == '*' && *(p + 1) == '*') {
+		if (*p == '*' && p[1] == '*') {
 			t.pos |= POS_SCANALL;
 			p += 2;
 		}
@@ -580,7 +586,7 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 			++p;
 		}
 		if (*p == 'x') {
-			t.pos |= POS_MARK_JUMP;
+			t.pos |= POS_JUMP;
 			++p;
 		}
 		if (*p == 'D') {
@@ -635,7 +641,7 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 				++p;
 			}
 		}
-		if (*p == 'r' && *(p + 1) == ':') {
+		if (*p == 'r' && p[1] == ':') {
 			t.pos |= POS_RELATION;
 			p += 2;
 			UChar* n = p;
@@ -665,6 +671,28 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 			t.pos |= POS_LEFTMOST;
 			++p;
 		}
+		if (*p == 'j') {
+			if (p[1] == 'M') {
+				t.pos |= POS_JUMP;
+				p += 2;
+			}
+			else if (p[1] == 'A') {
+				t.pos |= POS_JUMP;
+				t.jump_pos = JUMP_ATTACH;
+				p += 2;
+			}
+			else if (p[1] == 'T') {
+				t.pos |= POS_JUMP;
+				t.jump_pos = JUMP_TARGET;
+				p += 2;
+			}
+			else if (p[1] == 'C' && u_isdigit(p[2])) {
+				p += 2;
+				t.pos |= POS_JUMP;
+				t.jump_pos = SI8(*p - '0');
+				++p;
+			}
+		}
 	}
 
 	if (negative) {
@@ -677,7 +705,7 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 
 		size_t tries;
 		for (tries = 0; *p != ' ' && *p != '(' && tries < 100; ++tries) {
-			if (*p == '*' && *(p + 1) == '*') {
+			if (*p == '*' && p[1] == '*') {
 				t.offset_sub = GSR_ANY;
 				p += 2;
 			}
@@ -931,7 +959,7 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 			result->lines += SKIPWS(p);
 			Set* s = parseSetInlineWrapper(p);
 			t->cbarrier = s->hash;
-			AST_CLOSE(p);
+			AST_CLOSE_ID(p, s->hash);
 		}
 		result->lines += SKIPWS(p);
 		if (ux_simplecasecmp(p, STR_BARRIER)) {
@@ -940,7 +968,7 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 			result->lines += SKIPWS(p);
 			Set* s = parseSetInlineWrapper(p);
 			t->barrier = s->hash;
-			AST_CLOSE(p);
+			AST_CLOSE_ID(p, s->hash);
 		}
 		result->lines += SKIPWS(p);
 
@@ -966,7 +994,7 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 
 	if (linked) {
 		t->linked = parseContextualTestList(p, rule, in_tmpl);
-		if ((t->pos & POS_NONE) && !(t->linked->pos & POS_MARK_JUMP)) {
+		if (t->pos & POS_NONE) {
 			error("%s: Error: It does not make sense to LINK from a NONE test; perhaps you meant NOT or NEGATE on line %u near `%S`?\n", p);
 		}
 	}
@@ -976,8 +1004,6 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 		u_fflush(ux_stderr);
 	}
 
-	AST_CLOSE(p);
-
 	if (rule) {
 		if (rule->flags & RF_LOOKDELETED) {
 			t->pos |= POS_LOOK_DELETED;
@@ -991,9 +1017,15 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 	}
 
 	t = result->addContextualTest(ot);
+	if (profiler) {
+		profiler->addContext(t->hash, cur_grammar_n, cur_ast->b - cur_grammar, p - cur_grammar);
+	}
 	if (t->tmpl) {
 		deferred_tmpls[t] = tmpl_data;
 	}
+
+	AST_CLOSE_ID(p, t->hash);
+
 	return t;
 }
 
@@ -1024,11 +1056,11 @@ flags_t TextualParser::parseRuleFlags(UChar*& p) {
 	bool setflag = true;
 	while (setflag) {
 		setflag = false;
-		for (uint32_t i = 0; i < FLAGS_COUNT; i++) {
+		for (size_t i = 0; i < FLAGS_COUNT; i++) {
 			UChar* op = p;
 			if (ux_simplecasecmp(p, g_flags[i])) {
 				p += g_flags[i].size();
-				rv.flags |= (1 << i);
+				rv.flags |= (1ull << i);
 				setflag = true;
 
 				if (i == FL_SUB) {
@@ -1053,7 +1085,7 @@ flags_t TextualParser::parseRuleFlags(UChar*& p) {
 				// Rule flags followed by letters or valid set characters should not be flags.
 				if (*p != '(' && *p != ';' && !ISSPACE(*p)) {
 				undo_flag:
-					rv.flags &= ~(1 << i);
+					rv.flags &= ~(1ull << i);
 					p = op;
 					setflag = false;
 					break;
@@ -1070,6 +1102,10 @@ flags_t TextualParser::parseRuleFlags(UChar*& p) {
 				break;
 			}
 		}
+		// These flags must be the last flag, as a set will follow
+		if (rv.flags & (RF_WITHCHILD | RF_NOCHILD | RF_BEFORE | RF_AFTER)) {
+			break;
+		}
 	}
 
 	std::bitset<sizeof(rv.flags) * CHAR_BIT> bits;
@@ -1094,6 +1130,9 @@ flags_t TextualParser::parseRuleFlags(UChar*& p) {
 	if (rv.flags & RF_UNMAPLAST) {
 		rv.flags |= RF_UNSAFE;
 	}
+	if (rv.flags & RF_REMEMBERX) {
+		rv.flags |= RF_KEEPORDER;
+	}
 	if (rv.flags & RF_ENCL_FINAL) {
 		result->has_encl_final = true;
 	}
@@ -1130,7 +1169,7 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 		gbuffers[0][c] = 0;
 		Tag* wform = parseTag(&gbuffers[0][0], lp);
 		rule->wordform = wform;
-		AST_CLOSE(n);
+		AST_CLOSE_ID(n, wform->hash);
 	}
 
 	AST_OPEN(RuleType);
@@ -1226,6 +1265,9 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 	if (key == K_UNMAP && !(rule->flags & (RF_SAFE | RF_UNSAFE))) {
 		rule->flags |= RF_SAFE;
 	}
+	if (key == K_SETPARENT && !(rule->flags & (RF_SAFE | RF_UNSAFE))) {
+		rule->flags |= (safe_setparent ? RF_SAFE : RF_UNSAFE);
+	}
 
 	if (rule->flags & RF_WITHCHILD) {
 		AST_OPEN(RuleWithChildTarget);
@@ -1511,35 +1553,73 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 
 	if (!(rule->flags & RF_REMEMBERX)) {
 		bool found = false;
-		if (rule->dep_target && (rule->dep_target->pos & POS_MARK_JUMP)) {
+		if (rule->dep_target && (rule->dep_target->pos & POS_JUMP) && rule->dep_target->jump_pos == JUMP_MARK) {
 			found = true;
 		}
 		else {
 			for (auto it : rule->tests) {
-				if (it->pos & POS_MARK_JUMP) {
+				if ((it->pos & POS_JUMP) && it->jump_pos == JUMP_MARK) {
 					found = true;
 					break;
 				}
 			}
 			for (auto it : rule->dep_tests) {
-				if (it->pos & POS_MARK_JUMP) {
+				if ((it->pos & POS_JUMP) && it->jump_pos == JUMP_MARK) {
 					found = true;
 					break;
 				}
 			}
 		}
 		if (found) {
-			u_fprintf(ux_stderr, "%s: Warning: Rule on line %u had 'x' in the first part of a contextual test, but no REMEMBERX flag.\n", filebase, result->lines);
-			u_fflush(ux_stderr);
+			rule->flags |= RF_REMEMBERX | RF_KEEPORDER;
+		}
+	}
+
+	if (key == K_WITH) {
+		rule->flags |= RF_KEEPORDER;
+		result->lines += SKIPWS(p, '{', ';');
+		if (*p == '{') {
+			++p;
+			bool prev_in_nested_rule = in_nested_rule;
+			Rule* prev_nested_rule = nested_rule;
+			in_nested_rule = true;
+			nested_rule = rule;
+			AST_OPEN(RuleSubrules);
+			result->lines += SKIPWS(p);
+			do {
+				maybeParseRule(p);
+				result->lines += SKIPWS(p, '}', ';');
+				if (*p == ';') {
+					++p;
+					result->lines += SKIPWS(p);
+				}
+			} while (*p != '}');
+			++p;
+			AST_CLOSE(p);
+			nested_rule = prev_nested_rule;
+			in_nested_rule = prev_in_nested_rule;
 		}
 	}
 
 	rule->reverseContextualTests();
-	if (only_sets) {
-		result->destroyRule(rule);
+
+	bool destroy = only_sets;
+	UErrorCode status;
+	if (nrules) {
+		status = U_ZERO_ERROR;
+		uregex_setText(nrules, rule->name.c_str(), SI32(rule->name.size()), &status);
+		status = U_ZERO_ERROR;
+		if (!uregex_find(nrules, -1, &status)) {
+			destroy = true;
+		}
 	}
-	else {
-		addRuleToGrammar(rule);
+	if (nrules_inv) {
+		status = U_ZERO_ERROR;
+		uregex_setText(nrules_inv, rule->name.c_str(), SI32(rule->name.size()), &status);
+		status = U_ZERO_ERROR;
+		if (uregex_find(nrules_inv, -1, &status)) {
+			destroy = true;
+		}
 	}
 
 	result->lines += SKIPWS(p, ';');
@@ -1547,7 +1627,168 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 		u_fprintf(ux_stderr, "%s: Warning: Expected closing ; on line %u after previous rule!\n", filebase, result->lines);
 		u_fflush(ux_stderr);
 	}
-	AST_CLOSE(p);
+
+	if (destroy) {
+		result->destroyRule(rule);
+		AST_CLOSE(p);
+	}
+	else {
+		addRuleToGrammar(rule);
+
+		if (profiler) {
+			profiler->addRule(rule->number + 1, cur_grammar_n, cur_ast->b - cur_grammar, p - cur_grammar);
+		}
+		AST_CLOSE_ID(p, rule->number + 1);
+	}
+}
+
+bool TextualParser::maybeParseRule(UChar*& p) {
+	// ADDRELATIONS
+	if (IS_ICASE(p, "ADDRELATIONS", "addrelations")) {
+		parseRule(p, K_ADDRELATIONS);
+	}
+	// SETRELATIONS
+	else if (IS_ICASE(p, "SETRELATIONS", "setrelations")) {
+		parseRule(p, K_SETRELATIONS);
+	}
+	// REMRELATIONS
+	else if (IS_ICASE(p, "REMRELATIONS", "remrelations")) {
+		parseRule(p, K_REMRELATIONS);
+	}
+	// ADDRELATION
+	else if (IS_ICASE(p, "ADDRELATION", "addrelation")) {
+		parseRule(p, K_ADDRELATION);
+	}
+	// SETRELATION
+	else if (IS_ICASE(p, "SETRELATION", "setrelation")) {
+		parseRule(p, K_SETRELATION);
+	}
+	// REMRELATION
+	else if (IS_ICASE(p, "REMRELATION", "remrelation")) {
+		parseRule(p, K_REMRELATION);
+	}
+	// SETVARIABLE
+	else if (IS_ICASE(p, "SETVARIABLE", "setvariable")) {
+		parseRule(p, K_SETVARIABLE);
+	}
+	// REMVARIABLE
+	else if (IS_ICASE(p, "REMVARIABLE", "remvariable")) {
+		parseRule(p, K_REMVARIABLE);
+	}
+	// SETPARENT
+	else if (IS_ICASE(p, "SETPARENT", "setparent")) {
+		parseRule(p, K_SETPARENT);
+	}
+	// SETCHILD
+	else if (IS_ICASE(p, "SETCHILD", "setchild")) {
+		parseRule(p, K_SETCHILD);
+	}
+	// RESTORE
+	else if (IS_ICASE(p, "RESTORE", "restore")) {
+		parseRule(p, K_RESTORE);
+	}
+	// IFF
+	else if (IS_ICASE(p, "IFF", "iff")) {
+		parseRule(p, K_IFF);
+	}
+	// MAP
+	else if (IS_ICASE(p, "MAP", "map")) {
+		parseRule(p, K_MAP);
+	}
+	// ADD
+	else if (IS_ICASE(p, "ADD", "add")) {
+		parseRule(p, K_ADD);
+	}
+	// APPEND
+	else if (IS_ICASE(p, "APPEND", "append")) {
+		parseRule(p, K_APPEND);
+	}
+	// SELECT
+	else if (IS_ICASE(p, "SELECT", "select")) {
+		parseRule(p, K_SELECT);
+	}
+	// REMOVE
+	else if (IS_ICASE(p, "REMOVE", "remove")) {
+		parseRule(p, K_REMOVE);
+	}
+	// REPLACE
+	else if (IS_ICASE(p, "REPLACE", "replace")) {
+		parseRule(p, K_REPLACE);
+	}
+	// SUBSTITUTE
+	else if (IS_ICASE(p, "SUBSTITUTE", "substitute")) {
+		parseRule(p, K_SUBSTITUTE);
+	}
+	// COPY
+	else if (IS_ICASE(p, "COPY", "copy")) {
+		parseRule(p, K_COPY);
+			}
+	// UNMAP
+	else if (IS_ICASE(p, "UNMAP", "unmap")) {
+		parseRule(p, K_UNMAP);
+	}
+	// PROTECT
+	else if (IS_ICASE(p, "PROTECT", "protect")) {
+		parseRule(p, K_PROTECT);
+	}
+	// UNPROTECT
+	else if (IS_ICASE(p, "UNPROTECT", "unprotect")) {
+		parseRule(p, K_UNPROTECT);
+	}
+	// DELIMIT
+	else if (IS_ICASE(p, "DELIMIT", "delimit")) {
+		parseRule(p, K_DELIMIT);
+	}
+	// JUMP
+	else if (IS_ICASE(p, "JUMP", "jump")) {
+		parseRule(p, K_JUMP);
+	}
+	// MOVE
+	else if (IS_ICASE(p, "MOVE", "move")) {
+		parseRule(p, K_MOVE);
+	}
+	// SWITCH
+	else if (IS_ICASE(p, "SWITCH", "switch")) {
+		parseRule(p, K_SWITCH);
+	}
+	// EXECUTE
+	else if (IS_ICASE(p, "EXECUTE", "execute")) {
+		parseRule(p, K_EXECUTE);
+	}
+	// EXTERNAL
+	else if (IS_ICASE(p, "EXTERNAL", "external")) {
+		parseRule(p, K_EXTERNAL);
+	}
+	// REMCOHORT
+	else if (IS_ICASE(p, "REMCOHORT", "remcohort")) {
+		parseRule(p, K_REMCOHORT);
+	}
+	// ADDCOHORT
+	else if (IS_ICASE(p, "ADDCOHORT", "addcohort")) {
+		parseRule(p, K_ADDCOHORT);
+	}
+	// SPLITCOHORT
+	else if (IS_ICASE(p, "SPLITCOHORT", "splitcohort")) {
+		parseRule(p, K_SPLITCOHORT);
+	}
+	// MERGECOHORTS
+	else if (IS_ICASE(p, "MERGECOHORTS", "mergecohorts")) {
+		parseRule(p, K_MERGECOHORTS);
+	}
+	// RESTORE
+	else if (IS_ICASE(p, "RESTORE", "restore")) {
+		parseRule(p, K_RESTORE);
+	}
+	// WITH
+	else if (IS_ICASE(p, "WITH", "with")) {
+		parseRule(p, K_WITH);
+	}
+	else {
+		// we didn't see a rule
+		return false;
+	}
+	// the else clause didn't fire so we DID see a rule
+	return true;
 }
 
 void TextualParser::parseAnchorish(UChar*& p, bool rule_flags) {
@@ -1583,7 +1824,19 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 		u_fprintf(ux_stderr, "%s: Error: Input is empty - cannot continue!\n", fname);
 		CG3Quit(1);
 	}
+	if (profiler) {
+		parse_ast = true;
+	}
 
+	auto id = ++num_grammars;
+	if (profiler) {
+		UnicodeString str(input);
+		std::string utf8;
+		str.toUTF8String(utf8);
+		id = profiler->addGrammar(fname, utf8);
+	}
+	cur_grammar = input;
+	cur_grammar_n = id;
 	UChar* p = input;
 	result->lines = 1;
 	AST_OPEN(Grammar);
@@ -1619,7 +1872,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				if (*p != ';') {
 					error("%s: Error: Expected closing ; before line %u near `%S`!\n", p);
 				}
-				AST_CLOSE(p + 1);
+				AST_CLOSE_ID(p + 1, result->delimiters->hash);
 			}
 			// SOFT-DELIMITERS
 			else if (IS_ICASE(p, "SOFT-DELIMITERS", "soft-delimiters")) {
@@ -1645,7 +1898,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				if (*p != ';') {
 					error("%s: Error: Expected closing ; before line %u near `%S`!\n", p);
 				}
-				AST_CLOSE(p + 1);
+				AST_CLOSE_ID(p + 1, result->soft_delimiters->hash);
 			}
 			// TEXT-DELIMITERS
 			else if (IS_ICASE(p, "TEXT-DELIMITERS", "text-delimiters")) {
@@ -1681,7 +1934,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				if (*p != ';') {
 					error("%s: Error: Expected closing ; before line %u near `%S`!\n", p);
 				}
-				AST_CLOSE(p + 1);
+				AST_CLOSE_ID(p + 1, result->text_delimiters->hash);
 			}
 			// MAPPING-PREFIX
 			else if (IS_ICASE(p, "MAPPING-PREFIX", "mapping-prefix")) {
@@ -1748,7 +2001,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					Tag* t = parseTag(&gbuffers[0][0], p);
 					result->preferred_targets.push_back(t->hash);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, t->hash);
 					result->lines += SKIPWS(p);
 				}
 
@@ -1789,7 +2042,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					Tag* t = parseTag(&gbuffers[0][0], p);
 					result->reopen_mappings.insert(t->hash);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, t->hash);
 					result->lines += SKIPWS(p);
 				}
 
@@ -1869,70 +2122,6 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				}
 				AST_CLOSE(p + 1);
 			}
-			// ADDRELATIONS
-			else if (IS_ICASE(p, "ADDRELATIONS", "addrelations")) {
-				parseRule(p, K_ADDRELATIONS);
-			}
-			// SETRELATIONS
-			else if (IS_ICASE(p, "SETRELATIONS", "setrelations")) {
-				parseRule(p, K_SETRELATIONS);
-			}
-			// REMRELATIONS
-			else if (IS_ICASE(p, "REMRELATIONS", "remrelations")) {
-				parseRule(p, K_REMRELATIONS);
-			}
-			// ADDRELATION
-			else if (IS_ICASE(p, "ADDRELATION", "addrelation")) {
-				parseRule(p, K_ADDRELATION);
-			}
-			// SETRELATION
-			else if (IS_ICASE(p, "SETRELATION", "setrelation")) {
-				parseRule(p, K_SETRELATION);
-			}
-			// REMRELATION
-			else if (IS_ICASE(p, "REMRELATION", "remrelation")) {
-				parseRule(p, K_REMRELATION);
-			}
-			// SETVARIABLE
-			else if (IS_ICASE(p, "SETVARIABLE", "setvariable")) {
-				parseRule(p, K_SETVARIABLE);
-			}
-			// REMVARIABLE
-			else if (IS_ICASE(p, "REMVARIABLE", "remvariable")) {
-				parseRule(p, K_REMVARIABLE);
-			}
-			// SETPARENT
-			else if (IS_ICASE(p, "SETPARENT", "setparent")) {
-				parseRule(p, K_SETPARENT);
-			}
-			// SETCHILD
-			else if (IS_ICASE(p, "SETCHILD", "setchild")) {
-				parseRule(p, K_SETCHILD);
-			}
-			// EXTERNAL
-			else if (IS_ICASE(p, "EXTERNAL", "external")) {
-				parseRule(p, K_EXTERNAL);
-			}
-			// REMCOHORT
-			else if (IS_ICASE(p, "REMCOHORT", "remcohort")) {
-				parseRule(p, K_REMCOHORT);
-			}
-			// ADDCOHORT
-			else if (IS_ICASE(p, "ADDCOHORT", "addcohort")) {
-				parseRule(p, K_ADDCOHORT);
-			}
-			// SPLITCOHORT
-			else if (IS_ICASE(p, "SPLITCOHORT", "splitcohort")) {
-				parseRule(p, K_SPLITCOHORT);
-			}
-			// MERGECOHORTS
-			else if (IS_ICASE(p, "MERGECOHORTS", "mergecohorts")) {
-				parseRule(p, K_MERGECOHORTS);
-			}
-			// RESTORE
-			else if (IS_ICASE(p, "RESTORE", "restore")) {
-				parseRule(p, K_RESTORE);
-			}
 			// SETS
 			else if (IS_ICASE(p, "SETS", "sets")) {
 				p += 4;
@@ -1967,7 +2156,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					Tag* t = parseTag(&gbuffers[0][0], p);
 					tmp.insert(t->hash);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, t->hash);
 					result->lines += SKIPWS(p);
 				}
 
@@ -1981,11 +2170,16 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				list_tags.swap(tmp);
 				AST_CLOSE(p + 1);
 			}
-			// LIST
-			else if (IS_ICASE(p, "LIST", "list")) {
+			// LIST, OLIST
+			else if (IS_ICASE(p, "LIST", "list") || IS_ICASE(p, "OLIST", "olist")) {
 				Set* s = result->allocateSet();
 				s->line = result->lines;
+				bool ordered = false;
 				AST_OPEN(List);
+				if (*p == 'O') {
+					++p;
+					ordered = true;
+				}
 				p += 4;
 				result->lines += SKIPWS(p);
 				AST_OPEN(SetName);
@@ -2014,7 +2208,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					error("%s: Error: Encountered a %C before the expected = on line %u near `%S`!\n", *p, p);
 				}
 				++p;
-				parseTagList(p, s);
+				parseTagList(p, s, ordered);
 				s->rehash();
 				if (append) {
 					result->appendToSet(s);
@@ -2036,7 +2230,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				if (*p != ';') {
 					error("%s: Error: Expected closing ; before line %u near `%S`!\n", p);
 				}
-				AST_CLOSE(p + 1);
+				AST_CLOSE_ID(p + 1, s->hash);
 			}
 			// SET
 			else if (IS_ICASE(p, "SET", "set")) {
@@ -2094,7 +2288,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				if (*p != ';') {
 					error("%s: Error: Expected closing ; before line %u near `%S`! Probably caused by missing set operator.\n", p);
 				}
-				AST_CLOSE(p + 1);
+				AST_CLOSE_ID(p + 1, s->hash);
 			}
 			// MAPPINGS
 			else if (IS_ICASE(p, "MAPPINGS", "mappings")) {
@@ -2286,6 +2480,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					std::pair<UStringView, bool*>(STR_SELF_NO_BARRIER, &self_no_barrier),
 					std::pair<UStringView, bool*>(STR_ORDERED, &result->ordered),
 					std::pair<UStringView, bool*>(STR_ADDCOHORT_ATTACH, &result->addcohort_attach),
+					std::pair<UStringView, bool*>(STR_SAFE_SETPARENT, &safe_setparent),
 				};
 
 				while (*p != ';') {
@@ -2345,7 +2540,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 					Tag* t = parseTag(&gbuffers[0][0], p);
 					tmp.insert(t->hash);
 					p = n;
-					AST_CLOSE(p);
+					AST_CLOSE_ID(p, t->hash);
 					result->lines += SKIPWS(p);
 				}
 
@@ -2458,80 +2653,16 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				data.resize(read + 4 + 1);
 
 				uint32_t olines = 0;
-				swapper<uint32_t> oswap(true, olines, result->lines);
+				swapper oswap(true, olines, result->lines);
 				const char* obase = nullptr;
-				swapper<const char*> bswap(true, obase, filebase);
+				swapper bswap(true, obase, filebase);
+				UChar* cgram = nullptr;
+				swapper gswap(true, cgram, cur_grammar);
+				uint32_t ngram = 0;
+				swapper gnswap(true, ngram, cur_grammar_n);
 
 				parseFromUChar(&data[4], abspath.data());
 			}
-			// IFF
-			else if (IS_ICASE(p, "IFF", "iff")) {
-				parseRule(p, K_IFF);
-			}
-			// MAP
-			else if (IS_ICASE(p, "MAP", "map")) {
-				parseRule(p, K_MAP);
-			}
-			// ADD
-			else if (IS_ICASE(p, "ADD", "add")) {
-				parseRule(p, K_ADD);
-			}
-			// APPEND
-			else if (IS_ICASE(p, "APPEND", "append")) {
-				parseRule(p, K_APPEND);
-			}
-			// SELECT
-			else if (IS_ICASE(p, "SELECT", "select")) {
-				parseRule(p, K_SELECT);
-			}
-			// REMOVE
-			else if (IS_ICASE(p, "REMOVE", "remove")) {
-				parseRule(p, K_REMOVE);
-			}
-			// REPLACE
-			else if (IS_ICASE(p, "REPLACE", "replace")) {
-				parseRule(p, K_REPLACE);
-			}
-			// DELIMIT
-			else if (IS_ICASE(p, "DELIMIT", "delimit")) {
-				parseRule(p, K_DELIMIT);
-			}
-			// SUBSTITUTE
-			else if (IS_ICASE(p, "SUBSTITUTE", "substitute")) {
-				parseRule(p, K_SUBSTITUTE);
-			}
-			// COPY
-			else if (IS_ICASE(p, "COPY", "copy")) {
-				parseRule(p, K_COPY);
-			}
-			// JUMP
-			else if (IS_ICASE(p, "JUMP", "jump")) {
-				parseRule(p, K_JUMP);
-			}
-			// MOVE
-			else if (IS_ICASE(p, "MOVE", "move")) {
-				parseRule(p, K_MOVE);
-			}
-			// SWITCH
-			else if (IS_ICASE(p, "SWITCH", "switch")) {
-				parseRule(p, K_SWITCH);
-			}
-			// EXECUTE
-			else if (IS_ICASE(p, "EXECUTE", "execute")) {
-				parseRule(p, K_EXECUTE);
-			}
-			// UNMAP
-			else if (IS_ICASE(p, "UNMAP", "unmap")) {
-				parseRule(p, K_UNMAP);
-			}
-			// PROTECT
-			else if (IS_ICASE(p, "PROTECT", "protect")) {
-				parseRule(p, K_PROTECT);
-			}
-			// UNPROTECT
-			else if (IS_ICASE(p, "UNPROTECT", "unprotect")) {
-				parseRule(p, K_UNPROTECT);
-			}
 			// TEMPLATE
 			else if (IS_ICASE(p, "TEMPLATE", "template")) {
 				AST_OPEN(Template);
@@ -2599,13 +2730,13 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 						}
 					}
 					result->lines += SKIPTOWS(n, ')', true);
-					AST_CLOSE(n);
 					c = SI32(n - p);
 					u_strncpy(&gbuffers[0][0], p, c);
 					gbuffers[0][c] = 0;
 					left = parseTag(&gbuffers[0][0], p);
 					result->lines += SKIPWS(n);
 					p = n;
+					AST_CLOSE_ID(p, left->hash);
 
 					if (*p == ')') {
 						error("%s: Error: Encountered ) before the expected Right tag on line %u near `%S`!\n", p);
@@ -2620,13 +2751,13 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 						}
 					}
 					result->lines += SKIPTOWS(n, ')', true);
-					AST_CLOSE(n);
 					c = SI32(n - p);
 					u_strncpy(&gbuffers[0][0], p, c);
 					gbuffers[0][c] = 0;
 					right = parseTag(&gbuffers[0][0], p);
 					result->lines += SKIPWS(n);
 					p = n;
+					AST_CLOSE_ID(p, right->hash);
 
 					if (*p != ')') {
 						error("%s: Error: Encountered %C before the expected ) on line %u near `%S`!\n", *p, p);
@@ -2652,13 +2783,16 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 			}
 			// END
 			else if (IS_ICASE(p, "END", "end")) {
-				if (ISNL(*(p - 1)) || ISSPACE(*(p - 1))) {
-					if (*(p + 3) == 0 || ISNL(*(p + 3)) || ISSPACE(*(p + 3))) {
+				if (ISNL(p[-1]) || ISSPACE(p[-1])) {
+					if (p[3] == 0 || ISNL(p[3]) || ISSPACE(p[3])) {
 						break;
 					}
 				}
 				++p;
 			}
+			// Has to happen last, or things like MAPPINGS will be parsed as MAP PINGS
+			else if (maybeParseRule(p)) {
+			}
 			// No keyword found at this position, skip a character.
 			else {
 				UChar* n = p;
@@ -2686,7 +2820,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 		}
 	}
 
-	AST_CLOSE(p);
+	AST_CLOSE_ID(p, id);
 }
 
 int TextualParser::parse_grammar(const char* fname) {
@@ -2858,6 +2992,18 @@ int TextualParser::parse_grammar(UString& data) {
 		result->addTagToSet(t, set_c);
 		result->addSet(set_c);
 	}
+	// Create context sets for nested rules
+	{
+		constexpr UStringView grp[] = { STR_UU_C1, STR_UU_C2, STR_UU_C3, STR_UU_C4, STR_UU_C5, STR_UU_C6, STR_UU_C7, STR_UU_C8, STR_UU_C9 };
+		for (size_t i = 0; i < 9; i++) {
+			Set* set_c = result->allocateSet();
+			set_c->line = 0;
+			set_c->setName(grp[i]);
+			Tag* t = parseTag(grp[i]);
+			result->addTagToSet(t, set_c);
+			result->addSet(set_c);
+		}
+	}
 
 	parseFromUChar(&data[4], filename);
 
@@ -2955,6 +3101,14 @@ int TextualParser::parse_grammar(UString& data) {
 			orc->ors.push_back(unsafec);
 			orc = result->addContextualTest(orc);
 
+			if (profiler) {
+				Profiler::Key k{ET_CONTEXT, tmp->hash};
+				auto pc = profiler->entries.find(k);
+				if (pc != profiler->entries.end()) {
+					profiler->addContext(orc->hash, cur_grammar_n, pc->second.b, pc->second.e);
+				}
+			}
+
 			for (auto& cntx : result->contexts) {
 				if (cntx.second->linked == tmp) {
 					cntx.second->linked = orc;
@@ -2994,7 +3148,12 @@ void TextualParser::setVerbosity(uint32_t level) {
 }
 
 void TextualParser::addRuleToGrammar(Rule* rule) {
-	if (in_section) {
+	if (in_nested_rule) {
+		rule->section = -3;
+		result->addRule(rule);
+		nested_rule->sub_rules.push_back(rule);
+	}
+	else if (in_section) {
 		rule->section = SI32(result->sections.size()) - 1;
 		result->addRule(rule);
 	}
