@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -24,6 +22,7 @@
 #define c6d28b7452ec699b_TEXTUALPARSER_H
 
 #include "IGrammarParser.hpp"
+#include "Profiler.hpp"
 #include "Strings.hpp"
 #include "sorted_vector.hpp"
 
@@ -59,6 +58,7 @@ public:
 	const char* filebase = nullptr;
 	uint32SortedVector strict_tags;
 	uint32SortedVector list_tags;
+	Profiler* profiler = nullptr;
 
 private:
 	UChar nearbuf[32]{};
@@ -67,11 +67,16 @@ private:
 	uint32_t seen_mapping_prefix = 0;
 	flags_t section_flags;
 	bool option_vislcg_compat = false;
-	bool in_section = false, in_before_sections = true, in_after_sections = false, in_null_section = false;
+	bool in_section = false, in_before_sections = true, in_after_sections = false, in_null_section = false, in_nested_rule = false;
 	bool no_isets = false, no_itmpls = false, strict_wforms = false, strict_bforms = false, strict_second = false, strict_regex = false, strict_icase = false;
 	bool self_no_barrier = false;
+	bool safe_setparent = false;
 	bool only_sets = false;
+	Rule* nested_rule = nullptr;
 	const char* filename = nullptr;
+	UChar* cur_grammar = nullptr;
+	uint32_t cur_grammar_n = 0;
+	uint32_t num_grammars = 0;
 
 	typedef std::unordered_map<ContextualTest*, std::pair<size_t, UString>> deferred_t;
 	deferred_t deferred_tmpls;
@@ -86,7 +91,7 @@ private:
 	Tag* parseTag(const UStringView& to, const UChar* p = nullptr) {
 		return parseTag(to.data(), p);
 	}
-	void parseTagList(UChar*& p, Set* s);
+	void parseTagList(UChar*& p, Set* s, bool ordered = false);
 	Set* parseSet(const UChar* name, const UChar* p = nullptr);
 	Set* parseSetInline(UChar*& p, Set* s = nullptr);
 	Set* parseSetInlineWrapper(UChar*& p);
@@ -95,6 +100,7 @@ private:
 	void parseContextualTests(UChar*& p, Rule* rule);
 	void parseContextualDependencyTests(UChar*& p, Rule* rule);
 	flags_t parseRuleFlags(UChar*& p);
+	bool maybeParseRule(UChar*& p);
 	void parseRule(UChar*& p, KEYWORDS key);
 	void parseAnchorish(UChar*& p, bool rule_flags = true);
 
