@@ -1,22 +1,22 @@
 /*
- * Copyright (C) 2007-2021, GrammarSoft ApS
+ * Copyright (C) 2007-2023, GrammarSoft ApS
  * Developed by Tino Didriksen <mail@tinodidriksen.com>
  * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
  *
  * This file is part of VISL CG-3
  *
- * VISL CG-3 is free software: you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * VISL CG-3 is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this progam.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "stdafx.hpp"
@@ -25,7 +25,7 @@
 
 #include "version.hpp"
 
-using CG3::CG3Quit;
+using namespace CG3;
 
 #include <uoptions.hpp>
 namespace Options {
@@ -92,7 +92,7 @@ int main(int argc, char** argv) {
 	ucnv_setDefaultName("UTF-8");
 	uloc_setDefault("en_US_POSIX", &status);
 
-	CG3::MweSplitApplicator applicator(std::cerr);
+	MweSplitApplicator applicator(std::cerr);
 
 	applicator.verbosity_level = 0;
 	applicator.runGrammarOnText(std::cin, std::cout);
