@@ -1,22 +1,19 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2023, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
-* Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -35,12 +32,12 @@ namespace CG3 {
 template<typename T, typename V, T res_empty = T(-1), T res_del = T(-1) - 1>
 class flat_unordered_map {
 public:
-	typedef std::pair<const T, V> value_type;
-	typedef std::pair<T, V> value_type_real;
-	typedef T key_type;
-	typedef V mapped_type;
-	typedef typename std::vector<value_type_real> container;
-	typedef typename container::size_type size_type;
+	using value_type = std::pair<const T, V>;
+	using value_type_real = std::pair<T, V>;
+	using key_type = T;
+	using mapped_type = V;
+	using container = typename std::vector<value_type_real>;
+	using size_type = typename container::size_type;
 
 	class const_iterator {
 	private:
@@ -49,7 +46,11 @@ public:
 		size_t i;
 
 	public:
-		typedef value_type reference;
+		using iterator_category = std::bidirectional_iterator_tag;
+		using value_type = flat_unordered_map::value_type;
+		using difference_type = ptrdiff_t;
+		using pointer = value_type*;
+		using reference = value_type;
 
 		const_iterator()
 		  : fus(nullptr)
